@@ -1,12 +1,13 @@
-#!/usr/bin/env python
+# -*- Mode: Python -*-
 
-import exceptions
 import os
 import sys
 import select
 import unittest
 
-from common import glib
+import glib
+
+from compathelper import _bytes
 
 class TestMainLoop(unittest.TestCase):
     def testExceptionHandling(self):
@@ -27,11 +28,11 @@ class TestMainLoop(unittest.TestCase):
         glib.child_watch_add(pid, child_died, loop)
 
         os.close(pipe_r)
-        os.write(pipe_w, "Y")
+        os.write(pipe_w, _bytes("Y"))
         os.close(pipe_w)
 
         def excepthook(type, value, traceback):
-            assert type is exceptions.Exception
+            assert type is Exception
             assert value.args[0] == "deadbabe"
         sys.excepthook = excepthook
 
@@ -48,6 +49,3 @@ class TestMainLoop(unittest.TestCase):
         #
         sys.excepthook = sys.__excepthook__
         assert not got_exception
-
-if __name__ == '__main__':
-    unittest.main()
