@@ -7,7 +7,7 @@ import sys
 import os
 sys.path.insert(0, "../")
 
-from compathelper import _long, _unicode
+from compathelper import _long, _unicode, _bytes
 
 from gi.repository import GLib
 from gi.repository import GObject
@@ -345,6 +345,16 @@ class TestGdk(unittest.TestCase):
         self.assertEquals(color.green, 200)
         self.assertEquals(color.blue, 300)
 
+    def test_rgba(self):
+        self.assertEquals(Gdk.RGBA, overrides.Gdk.RGBA)
+        rgba = Gdk.RGBA(0.1, 0.2, 0.3, 0.4)
+        self.assertEquals(rgba.red, 0.1)
+        self.assertEquals(rgba.green, 0.2)
+        self.assertEquals(rgba.blue, 0.3)
+        self.assertEquals(rgba.alpha, 0.4)
+        rgba.green = 0.9
+        self.assertEquals(rgba.green, 0.9)
+
     def test_event(self):
         event = Gdk.Event.new(Gdk.EventType.CONFIGURE)
         self.assertEquals(event.type, Gdk.EventType.CONFIGURE)
@@ -359,6 +369,51 @@ class TestGdk(unittest.TestCase):
         event.type = Gdk.EventType.SCROLL
         self.assertRaises(AttributeError, lambda: getattr(event, 'foo_bar'))
 
+    def test_event_structures(self):
+        def button_press_cb(button, event):
+            self.assertTrue(isinstance(event, Gdk.EventButton))
+            self.assertTrue(event.type == Gdk.EventType.BUTTON_PRESS)
+            self.assertEquals(event.send_event, 0)
+            self.assertEquals(event.get_state(), Gdk.ModifierType.CONTROL_MASK)
+            self.assertEquals(event.get_root_coords(), (2, 5))
+
+            event.time = 12345
+            self.assertEquals(event.get_time(), 12345)
+
+        w = Gtk.Window()
+        b = Gtk.Button()
+        b.connect('button-press-event', button_press_cb)
+        w.add(b)
+        w.show_all()
+        Gdk.test_simulate_button(b.get_window(),
+                                 2, 5,
+                                 0,
+                                 Gdk.ModifierType.CONTROL_MASK,
+                                 Gdk.EventType.BUTTON_PRESS)
+
+    def test_cursor(self):
+        self.assertEquals(Gdk.Cursor, overrides.Gdk.Cursor)
+        c = Gdk.Cursor(Gdk.CursorType.WATCH)
+        self.assertNotEqual(c, None)
+        c = Gdk.Cursor(cursor_type = Gdk.CursorType.WATCH)
+        self.assertNotEqual(c, None)
+
+        display_manager = Gdk.DisplayManager.get()
+        display = display_manager.get_default_display()
+
+        test_pixbuf = GdkPixbuf.Pixbuf.new(GdkPixbuf.Colorspace.RGB,
+                                           False,
+                                           8,
+                                           5,
+                                           10)
+
+        c = Gdk.Cursor(display,
+                       test_pixbuf,
+                       y=0, x=0)
+
+        self.assertNotEqual(c, None)
+        self.assertRaises(ValueError, Gdk.Cursor, 1, 2, 3)
+
 class TestGtk(unittest.TestCase):
 
     def test_container(self):
@@ -632,7 +687,20 @@ class TestGtk(unittest.TestCase):
         test_pyobj = TestPyObject()
         test_pydict = {1:1, "2":2, "3":"3"}
         test_pylist = [1,"2", "3"]
-        tree_store = Gtk.TreeStore(int, 'gchararray', TestGtk.TestClass, object, object, object)
+        tree_store = Gtk.TreeStore(int,
+                                   'gchararray',
+                                   TestGtk.TestClass,
+                                   object,
+                                   object,
+                                   object,
+                                   bool,
+                                   bool,
+                                   GObject.TYPE_UINT,
+                                   GObject.TYPE_ULONG,
+                                   GObject.TYPE_INT64,
+                                   GObject.TYPE_UINT64,
+                                   GObject.TYPE_UCHAR,
+                                   GObject.TYPE_CHAR)
 
         parent = None
         for i in range(100):
@@ -643,7 +711,16 @@ class TestGtk(unittest.TestCase):
                                                 testobj,
                                                 test_pyobj,
                                                 test_pydict,
-                                                test_pylist))
+                                                test_pylist,
+                                                i % 2,
+                                                bool(i % 2),
+                                                i,
+                                                9223372036854775808,
+                                                -9223372036854775808,
+                                                0xffffffffffffffff,
+                                                254,
+                                                _bytes('a')
+                                                ))
 
         # len gets the number of children in the root node
         # since we kept appending to the previous node
@@ -656,9 +733,6 @@ class TestGtk(unittest.TestCase):
 
         treeiter = tree_store.iter_children(parent)
         while treeiter:
-           i = tree_store.get_value(treeiter, 0)
-           s = tree_store.get_value(treeiter, 1)
-           obj = tree_store.get_value(treeiter, 2)
            i = tree_store.get_value(treeiter, 0)
            s = tree_store.get_value(treeiter, 1)
            obj = tree_store.get_value(treeiter, 2)
@@ -671,6 +745,25 @@ class TestGtk(unittest.TestCase):
            pylist = tree_store.get_value(treeiter, 5)
            self.assertEquals(pylist, test_pylist)
 
+           bool_1 = tree_store.get_value(treeiter, 6)
+           bool_2 = tree_store.get_value(treeiter, 7)
+           self.assertEquals(bool_1, bool_2)
+           self.assertTrue(isinstance(bool_1, bool))
+           self.assertTrue(isinstance(bool_2, bool))
+
+           uint_ = tree_store.get_value(treeiter, 8)
+           self.assertEquals(uint_, i)
+           ulong_ = tree_store.get_value(treeiter, 9)
+           self.assertEquals(ulong_, 9223372036854775808)
+           int64_ = tree_store.get_value(treeiter, 10)
+           self.assertEquals(int64_, -9223372036854775808)
+           uint64_ = tree_store.get_value(treeiter, 11)
+           self.assertEquals(uint64_, 0xffffffffffffffff)
+           uchar_ = tree_store.get_value(treeiter, 12)
+           self.assertEquals(ord(uchar_), 254)
+           char_ = tree_store.get_value(treeiter, 13)
+           self.assertEquals(char_, 'a')
+
            parent = treeiter
            treeiter = tree_store.iter_children(parent)
 
@@ -684,7 +777,7 @@ class TestGtk(unittest.TestCase):
         test_pydict = {1:1, "2":2, "3":"3"}
         test_pylist = [1,"2", "3"]
 
-        list_store = Gtk.ListStore(int, str, 'GIOverrideTreeAPITest', object, object, object)
+        list_store = Gtk.ListStore(int, str, 'GIOverrideTreeAPITest', object, object, object, bool, bool)
         for i in range(93):
             label = 'this is row #%d' % i
             testobj = TestGtk.TestClass(self, i, label)
@@ -693,7 +786,9 @@ class TestGtk(unittest.TestCase):
                                         testobj,
                                         test_pyobj,
                                         test_pydict,
-                                        test_pylist))
+                                        test_pylist,
+                                        i % 2,
+                                        bool(i % 2)))
 
         i = 93
         label = _unicode('this is row #93')
@@ -704,6 +799,8 @@ class TestGtk(unittest.TestCase):
         list_store.set_value(treeiter, 3, test_pyobj)
         list_store.set_value(treeiter, 4, test_pydict)
         list_store.set_value(treeiter, 5, test_pylist)
+        list_store.set_value(treeiter, 6, 1)
+        list_store.set_value(treeiter, 7, True)
 
         # test automatic unicode->str conversion
         i = 94
@@ -713,17 +810,29 @@ class TestGtk(unittest.TestCase):
                                       TestGtk.TestClass(self, i, label),
                                       test_pyobj,
                                       test_pydict,
-                                      test_pylist))
+                                      test_pylist,
+                                      0,
+                                      False))
 
         # add sorted items out of order to test insert* apis
+        # also test sending in None to not set a column
         i = 97
         label = 'this is row #97'
-        treeiter = list_store.append((i,
-                                      label,
-                                      TestGtk.TestClass(self, i, label),
+        treeiter = list_store.append((None,
+                                      None,
+                                      None,
                                       test_pyobj,
-                                      test_pydict,
-                                      test_pylist))
+                                      None,
+                                      test_pylist,
+                                      1,
+                                      None))
+
+        list_store.set_value(treeiter, 0, i)
+        list_store.set_value(treeiter, 1, label)
+        list_store.set_value(treeiter, 2, TestGtk.TestClass(self, i, label))
+        list_store.set_value(treeiter, 4, test_pydict)
+        list_store.set_value(treeiter, 7, True)
+ 
         # this should append
         i = 99
         label = 'this is row #99'
@@ -732,7 +841,9 @@ class TestGtk(unittest.TestCase):
                                  TestGtk.TestClass(self, i, label),
                                  test_pyobj,
                                  test_pydict,
-                                 test_pylist))
+                                 test_pylist,
+                                 1,
+                                 True))
 
         i = 96
         label = 'this is row #96'
@@ -741,7 +852,9 @@ class TestGtk(unittest.TestCase):
                                             TestGtk.TestClass(self, i, label),
                                             test_pyobj,
                                             test_pydict,
-                                            test_pylist))
+                                            test_pylist,
+                                            0,
+                                            False))
 
         i = 98
         label = 'this is row #98'
@@ -750,7 +863,9 @@ class TestGtk(unittest.TestCase):
                                            TestGtk.TestClass(self, i, label),
                                            test_pyobj,
                                            test_pydict,
-                                           test_pylist))
+                                           test_pylist,
+                                           0,
+                                           False))
 
 
         i = 95
@@ -760,7 +875,9 @@ class TestGtk(unittest.TestCase):
                                TestGtk.TestClass(self, i, label),
                                test_pyobj,
                                test_pydict,
-                               test_pylist))
+                               test_pylist,
+                               1,
+                               True))
 
         self.assertEquals(len(list_store), 100)
 
@@ -782,6 +899,13 @@ class TestGtk(unittest.TestCase):
             self.assertEquals(pydict, test_pydict)
             pylist = list_store.get_value(treeiter, 5)
             self.assertEquals(pylist, test_pylist)
+
+            bool_1 = list_store.get_value(treeiter, 6)
+            bool_2 = list_store.get_value(treeiter, 7)
+            self.assertEquals(bool_1, bool_2)
+            self.assertTrue(isinstance(bool_1, bool))
+            self.assertTrue(isinstance(bool_2, bool))
+
             treeiter = list_store.iter_next(treeiter)
 
             counter += 1
@@ -1180,6 +1304,14 @@ class TestGtk(unittest.TestCase):
         self.assertEquals(table.get_size(), (2,3))
         self.assertEquals(table.get_homogeneous(), True)
 
+        # Test PyGTK interface
+        table = Gtk.Table(rows=3, columns=2)
+        self.assertEquals(table.get_size(), (3,2))
+        # Test using the actual property names
+        table = Gtk.Table(n_rows=2, n_columns=3, homogeneous=True)
+        self.assertEquals(table.get_size(), (2,3))
+        self.assertEquals(table.get_homogeneous(), True)
+
         label = Gtk.Label('Hello')
         table.attach(label, 0, 1, 0, 1)
         self.assertEquals(label, table.get_children()[0])
@@ -1324,3 +1456,11 @@ class TestGio(unittest.TestCase):
         self.assertEqual(len(empty), 0)
         self.assertEqual(bool(empty), True)
         self.assertEqual(empty.keys(), [])
+
+    def test_closures(self):
+        # make sure this doesn't crash
+        def fake_cb(*args):
+            pass
+
+        ag = Gtk.AccelGroup()
+        ag.connect(Gdk.KEY_l, Gdk.ModifierType.CONTROL_MASK, 0, fake_cb)
